! Kolmog.x <in.H >out.H
! Resting in, and maybe modified by ~/brad/Passive , fall 2001
! Puts data on a helix and factorizes it with Kolmogoroff
!
!   On input: 
!   1-axis     x
!   2-axis     y
!   3-axis     time
!   4-axis     multiple records (to stack elsewhere)
!
!   center=1    centers the impulse response
!   auto=0      gives acf instead of min phase factor
!
program kolmog
  use sep
  implicit none

  integer :: n1,n2,n3,i3,n4,i4
  integer :: n123,n123pad,n12
  integer :: jpad2
  integer :: center,auto
  real    :: d1,d2
  real, allocatable, dimension(:)    :: data
  complex, allocatable, dimension(:) :: cdata

  call sep_init(SOURCE)
  call from_param('center',center,1)
  call from_param('auto',auto,0)
  call from_history(n1,n2,n3)
  call from_history('n4',n4,1)
  call from_history('d1',d1)
  call from_history('d2',d2)
  if (center==1) then
     call to_history('o1',-((n1-1)/2)*d1)
     call to_history('o2',-((n2-1)/2)*d2)
  end if
  call sep_close()

  n123=n1*n2*n3
  n12=n1*n2
  n123pad=jpad2(n123)
  write(0,*) 'n123=',n123,'   (',real(n123pad*8)/real(1024*1024),'Mbytes)'

  allocate(data(n12),cdata(n123pad))

  records: do i4=1,n4
     write(0,*) '  reading data panel: ',i4
     cdata=0.
     do i3=1,n3  !make data complex
        call sep_read(data)
        cdata((i3-1)*n12+1:(i3-1)*n12+n12)=cmplx(data(1:n12),0.)
     end do
     write(0,*) '    calling ftu...'
     call jftu( 1., n123pad, cdata)
     cdata=cdata*conjg(cdata)

     if (auto==1) then
        write(0,*) '    calling inverse ftu...'
        call jftu(-1., n123pad, cdata)
     else
        write(0,*) '    calling kolmog...'
        call kolmogoroff(n123pad,cdata)
     end if

     do i3=1,n3
        if (center==0) then
           data(1:n1*n2)=cdata((i3-1)*n3+1:(i3-1)*n3+n1*n2)
        else
           center=(n2/2-1)*n1 + (n1/2-1)
           if (i3==1) then
              data(1:center)=0.
              data(1+center:n12)=cdata(1:n12-center)
           else
              data(1:n12)=cdata( (i3-1)*n12+1 -center:(i3-1)*n12+n12 -center)
           end if
        end if
        call sep_write(data)
     end do
  end do records
end program kolmog

subroutine kolmogoroff( n, cx)  ! Spectral factorization (from pvi)
  integer i,              n       ! input:  cx = spectrum
  complex cx(n)                   ! output: cx = min phase wavelet
  cx(1:n) = clog( cx(1:n) )
  call jftu( -1., n, cx)
  cx(1:n) = cx(1:n)*sqrt(1./n)
  cx(1    ) = cx(1    ) / 2.
  cx(1+n/2) = cx(1+n/2) / 2.
  cx(1+n/2+1:n) = 0.
  call jftu( +1., n, cx) 
  cx(1:n) = cx(1:n)*sqrt(1.*n)
  cx(1:n) = cexp( cx(1:n))
  call jftu( -1., n, cx)
  cx(1:n) = cx(1:n)*sqrt(1./n)
end subroutine kolmogoroff

subroutine jftu( signi, nx, cx )
!   complex fast fourier transform with unitary scaling
!
!               1         nx          signi*2*pi*i*(j-1)*(k-1)/nx
!   cx(k)  =  -------- * sum cx(j) * e
!             sqrt(nx)   j=1             for k=1,2,...,nx=2**integer
!
  integer :: nx, i, j, k, m, istep, jpad2
  real    :: signi
  real*8  :: arg,pi
  complex :: cx(nx), cw, cdel, ct
  pi=4.0d0*datan(1.0d0)
  if( nx /= jpad2(nx) )  call erexit('ftu: nx not a power of 2')
  cx = cx/sqrt( 1.*nx)
  j = 1;  k = 1
  do i= 1, nx
     if (i<=j) then
        ct = cx(j); cx(j) = cx(i); cx(i) = ct
     end if
     m = nx/2
     do while (j>m .and. m>1) 
        j = j-m; m = m/2 
     end do
     j = j+m
  end do
  do 
     istep = 2*k;   cw = 1.;   arg = signi*pi/k
     cdel = cmplx( cos(arg), sin(arg))
     do m= 1, k
        do i= m, nx, istep
           ct=cw*cx(i+k);  cx(i+k)=cx(i)-ct;  cx(i)=cx(i)+ct
        end do
        cw = cw * cdel
     end do
     k = istep
     if (k>=nx) exit
  end do
end subroutine jftu

integer function jpad2(n)
  integer n
  jpad2=1
  do
     if (jpad2.ge.n) exit
     jpad2=jpad2*2
  end do
end function jpad2




  
  








