!!$
!!$=head1 NAME
!!$
!!$ Lloyd_vel - Select reference velocities 
!!$
!!$=head1 SYNOPSIS
!!$
!!$Lloyd_vel  < vel.H >out.H
!!$
!!$
!!$=over 4
!!$
!!$
!!$=item nref (integer)
!!$
!!$  Number of reference velocities
!!$
!!$
!!$=item min_region_pct (real)
!!$
!!$  [1.]  Minimum percentage of the velocity field associated with a given reference
!!$
!!$=item min_dev 
!!$
!!$  [6.]  Minimum deviation (in percent) between reference velocities
!!$
!!$=item niter_lloyd 
!!$
!!$  [30] Number of iteratons of the lloyd to run
!!$
!!$=item When doing offset gathers  ahy_n,ahy_d,ahy_o = axis 4 of D and R
!!$
!!$=item nzs must be a multiple of az__n
!!$
!!$=item nws*nmpi_threads*nomp_threads must be a multiple of aw__n
!!$
!!$=cut
!!$
!!$

program lloyd
use sep3d_struct_mod
use lloyd_mod
use sep
implicit none
type(sep3d) :: attr_s
real, allocatable :: out(:,:),array(:,:),data(:,:,:)
integer,allocatable :: iregion(:),nstart(:)
integer :: nref,iref
integer::n(6),ndim,i,j,k,l,nn,step,verb
real:: o(6),d(6)
real:: power,mval,nval,d_amp
character(len=20)::pickfile

call sep_init(SOURCE)
call init_sep3d("in",attr_s,"INPUT")
n=1;o=0.;d=0.
ndim=sep3d_ndims(attr_s)
n(1:sep3d_ndims(attr_s))=attr_s%n(1:sep3d_ndims(attr_s))
d(1:sep3d_ndims(attr_s))=attr_s%d(1:sep3d_ndims(attr_s))
o(1:sep3d_ndims(attr_s))=attr_s%o(1:sep3d_ndims(attr_s))

allocate(nstart(ndim))
call from_param("nstart",nstart)
call from_param("power",power,1.)
call from_param("steps",step,10)
call from_param("verb",verb,2)
call from_param("pickfile",pickfile,'picks')
write(0,*) 'output pick file: ',trim(pickfile)

!allocate(scales(n(1)))
!scales=1
!call from_param("scales",scales,scales)

nref=product(nstart)
call to_history("n1",nref)
call to_history("n2",ndim)

call init_lloyd()

write(0,*) "nref",nref
write(0,*) 'ndim',ndim
write(0,*) 'n',n(1:ndim)
write(0,*) 'd',d(1:ndim)
write(0,*) 'o',o(1:ndim)

allocate(data(n(1),n(2),n(3)))
call sep_read(data)

!!make data into something like a histogram. High amplitude gets more locations.
!mval=maxval(data)
!nval=minval(data)
!data=data/((mval-nval)/2+nval)
data=data**2
mval=maxval(data)
power=mval*power/100
d_amp=(mval-power)/step
write(0,*) 'mval',mval
write(0,*) 'power',power
write(0,*) 'unit',d_amp

!$OMP PARALLEL DO PRIVATE(i,j,k)
do i=1,n(3)
   do j=1,n(2)
      do k=1,n(1)
         if(data(k,j,i)>power) then
            data(k,j,i)=int(0.5+(data(k,j,i)-power)/d_amp)
         else
            data(k,j,i)=0
         end if
      end do
   end do
end do
!$OMP END PARALLEL DO 

nn=int(sum(data));write(0,*) '  number of data vector coordinates :',nn

allocate(array(nn,ndim))
nn=0
!!make real data vector
!!$OMP PARALLEL DO PRIVATE(i,j,k,nn,step)
do i=1,n(3)
   do j=1,n(2)
      do k=1,n(1)
         if(data(k,j,i)>power) then
            step=int(data(k,j,i))
            do l=1,step
               nn=nn+1
               array(nn,1)=o(1)+(k-1)*d(1)
               array(nn,2)=o(2)+(j-1)*d(2)
               array(nn,3)=o(3)+(i-1)*d(3)
            end do
         end if
      end do
   end do
end do

!!$OMP END PARALLEL DO 
deallocate(data)
allocate(iregion(nn), out(nref,ndim))

iref= lloyd_go(array, out, nref, iregion,nstart)

nref=min(iref,nref)
call to_history("n1",iref)
call srite("out",out(1:nref,:),ndim*nref*4)

OPEN (UNIT=8, FILE=trim(pickfile),STATUS='UNKNOWN', ACTION='WRITE',IOSTAT=i)
write(0,*) ''
if(ndim==3) then
   if(verb>6)then
      do k=1,nref
         write(0,'(i2,a,3f13.8,2i4,f6.1,2i5)') k,':',out(k,:),0,0,1.0,10
      end do
   end if
   do k=1,nref
         write(8,'(a,3f13.8,2i4,f6.1,2i5)') 'x',out(k,:),0,0,1.0,10
   end do
end if

if(ndim==2) then
   if(verb>6)then
      do k=1,nref
         write(0,'(i2,a,2f13.8,3i4,f6.1,2i5)') k,':',out(k,:),0,0,0,1.0,10
      end do
   end if
   do k=1,nref
      write(8,'(a,2f13.8,3i4,f6.1,2i5)') 'x',out(k,:),0,0,0,1.0,10
   end do
end if
if(ndim==4) then
   if(verb>6)then
      do k=1,nref
         write(0,'(i2,a,4f13.8,i4,f6.1,2i5)') k,':',out(k,:),0,1.0,10
      end do
   end if
   do k=1,nref
      write(8,'(a,4f13.8,i4,f6.1,2i5)') 'x',out(k,:),0,1.0,10
   end do
end if

CLOSE(UNIT=8)

end program lloyd
