program SlantStackTimeInversion

  use sep
  !solver mods
  use solver_tiny_brad
  use cgstep_mod
  !weights and regularizors
  !  use identity_mod
  !  use weighthrt
  !model operators
  use pxpytime

  implicit none
  real   ,pointer,dimension(:)  :: dat
  real   ,pointer,dimension(:)  :: modl
  real   ,pointer,dimension(:,:):: glocs 

  integer    :: nt,ntm,np(2),esize,ntr,nh,n2,previts,prevtrcs,st
  real       :: dt,dtm,dp(2),center
  real       :: ot,otm,op(2)

  logical    :: adj,add,para

  !----------------------------------------------------------------
  integer                          :: niter,its, nmod,ndat,i,nter,blocks,jniter,done
  integer                          :: mmax, l1, verb, j, bound
  real                             :: gamd, gamx, eps, nu, mu, accuracy
  real                             :: low, up
  real, dimension (:), allocatable :: l, u
  real, dimension (:), pointer     :: mask
  integer                          :: weight,nq
  !----------------------------------------------------------------

  !! File setup 
  !need a geometry file w/ locations of geophones

  call sep_init(SOURCE)
  call getch_add_string("head=/dev/null")
  call getch_add_string("noheader=y")
  call set_no_putch()

  call from_aux("geom","n1",nh); call from_aux("geom","n2",ntr)
  call from_param('adj',adj,.true.)
  call from_param('add',add,.false.)
  call from_param('para',para,.false.)
  if(adj) then
     call auxin("data")
     call from_aux('data',"n1",nt); call from_aux('data',"d1",dt);call from_aux('data',"o1",ot)
     call from_aux('data','esize',esize) ; if (esize==8) call seperr('Time damnit!')

     if (add) then
        call auxinout("model")
        call from_aux('model',"n1",ntm  );call from_aux('model',"d1",dtm  );call from_aux('model',"o1",otm)
        call from_aux('model',"n2",np(1));call from_aux('model',"d2",dp(1));call from_aux('model',"o2",op(1))
        call from_aux('model',"n3",np(2));call from_aux('model',"d3",dp(2));call from_aux('model',"o3",op(2))
        if (ntm.ne.nt.or.dtm.ne.dt.or.otm.ne.ot) then
           call seperr("Bad time axes between model and data volumes")
        end if
        if(.not.para) then
           call from_aux("model","iterations",previts,0)
        else
           call from_aux("model","previous_data_traces",prevtrcs,0)
        end if
     else
        call auxout("model") ; 
        call from_param("npx",np(1),100);         call from_param("npy",np(2),np(1))!100); 
        call from_param("dpx",dp(1),0.00001333);  call from_param("dpy",dp(2),dp(1))!0.00001333)
        call from_param("opx",op(1),-0.000666667);call from_param("opy",op(2),op(1))!-0.000666667)
        call to_history("n1",nt   ,"model")
        call to_history("n2",np(1),"model");
        call to_history("n3",np(2),"model")
        call to_history("d1",dt   ,"model")
        call to_history("d2",dp(1),"model");
        call to_history("d3",dp(2),"model")
        call to_history("o1",ot   ,"model")
        call to_history("o2",op(1),"model");
        call to_history("o3",op(2),"model")
        call to_history("n4",1,"model")
     end if

     if (exist_file ("recons")) then 
        call auxout("recons")
        call to_history ("o1", ot,"recons")
        call to_history ("d1", dt,"recons")
        call to_history ("n1", nt,"recons")
        call to_history ("label1","time(s)","recons")

        call to_history ("o2", 0.,"recons")
        call to_history ("d2", 1.,"recons")
        call to_history ("n2", ntr,"recons")

        call to_history ("o3", 0.,"recons")
        call to_history ("d3", 1.,"recons")
        call to_history ("n3", 1,"recons")
        call to_history ("label2","traces","recons")
        call to_history("n4",1,"recons")
     end if
!     call copy_history("data","model")
  else
     call auxin("model")
     call from_aux("model","o1",otm)
     call from_aux("model","n1",ntm)  ;call from_aux("model","d1",dtm);
     call from_aux("model","n2",np(1));call from_aux("model","n3",np(2))
     call from_aux("model","d2",dp(1));call from_aux("model","d3",dp(2))
     call from_aux("model","o2",op(1));call from_aux("model","o3",op(2))
     if (add) then
        call auxinout("data")
        call from_aux('data',"n1",ntm  );call from_aux('data',"d1",dt  );call from_aux('data',"o1",ot)
        if (ntm.ne.nt.or.dtm.ne.dt.or.otm.ne.ot) then
           call seperr("Bad time axes between model and data volumes")
        end if
     else
        call auxout("data")
        nt=ntm;dt=dtm;ot=otm
        call to_history("n1",ntm,"data");call to_history("d1",dtm,"data");call to_history("o1",otm,"data")
        call to_history("n2",ntr,"data");call to_history("d2",1.,"data");call to_history("o2",0.,"data")
        call to_history ("label2","traces","data");call to_history("n3",1,"data")
     end if
  end if

  !! inversion parameters
  call from_param ("niter",niter,10)
  call from_param ('jniter',jniter,niter)
  call from_param ("verb",verb,2)
  call from_param ("eps",eps,0.)


  if (verb>0)write(0,*) ' '
  if (verb>0)write(0,*) '*************************************************'
  if (verb>0)write(0,*) 'Data domain'
  if (verb>0)write(0,'(a,i6,a,i6)') '    ntraces:',ntr,'  nt:',nt
  if (verb>0)write(0,*) '*************************************************'
  if (verb>0)write(0,*) 'Model domain'
  if (verb>0)write(0,*) '        n        d           o'
  if (verb>0)write(0,'(a,i4,f12.7,f12.7)') '    px:',np(1),dp(1),op(1)
  if (verb>0)write(0,'(a,i4,f12.7,f12.7)') '    py:',np(2),dp(2),op(2)
  if (verb>0)write(0,*) '          adj:',adj,'   add:',add
  if (verb>0)write(0,*) '*************************************************'
  if(adj) then
     if (verb>0)write(0,*) "Adjoint transform: seismic traces to tau-p domain"
     if (verb>0)write(0,*) 'Inversion parameters'
     if (verb>0)write(0,*) '    niter =:',niter
     if (verb>0)write(0,*) '    eps   =:',eps
     if (verb>0)write(0,*) '    verb  =:',verb
     if (verb>0.and.previts.gt.0)write(0,'(a,i4)') '     Previous iterations:',previts
     if (verb>0.and.previts.gt.0)write(0,'(a,i4)') '     Previous data traces:',prevtrcs
  else
     if (verb>0) write(0,*)  "Forward transform: tau-p to seismic traces"
  end if
  write(0,*) '*************************************************'
  !----------------------------------------------------------------

  allocate(glocs(nh,ntr))
  call sep_read(glocs,"geom")
  call slantstack3d_init(glocs,nt,ntr,np,dt,dp,ot,op,para)

  !! allocation & reading
  nmod=nt*np(1)*np(2)
  ndat=nt*ntr
  allocate( dat(ndat) );dat=0.
  allocate(modl(nmod) );modl=0.

  !!prep. work
  prep:if(adj)then
     if (exist_file("recons")) then
        call sep_write(dat,'recons')
        call sseek('recons',0,0)
     end if
     call sep_read(dat,'data')
     if(add) then 
        call sep_read(modl,'model') 
     else
        call sep_write(modl,'model')
     end if
     call sseek('model',0,0)
  else
     call sep_read(modl,'model')
     if(add) then
        call sep_read(dat,'data')
     else
        call sep_write(dat,'data')
     end if
     call sseek('data',0,0)
  end if prep

  call sep_begin_prog()

  !! Do the deed
  invert:if(adj) then
     write(0,'(a,f4.1)') ' Iteration blocks: ', real(niter)/real(jniter)
     call solver(modl,dat,slantstack3d_lop,cgstep,add,niter,jniter)

     if(.not.para) then
        call to_history("iterations",niter+previts,"model")
     else
        call to_history("previous_data_traces",ntr+prevtrcs,"model")
     end if
     if (exist_file("recons")) then
        st = slantstack3d_lop(.false.,.false.,modl,dat)
        call sep_write(dat,"recons")
     end if
  else
     st = slantstack3d_lop(.false.,.false.,modl,dat)
     call sep_write(dat,'data')
  end if invert

  call cgstep_close ( )
  call sep_close()
  deallocate(modl,dat,glocs)
  if (exist_file("mask")) deallocate(mask)
  write(0,*) 'Finished ========='
  call sep_end_prog()
  call exit (0)

end program SlantStackTimeInversion

