module pxpytime
  use sep
  implicit none
  integer,private    :: npx,npy,ntr,nt !x is first dimension
  real   ,private    :: dt,dpx,dpy
  real   ,private    :: ot,opx,opy
  real   ,private    :: xcen,ycen,pi
  real   ,private,dimension(:,:),pointer:: glocs
contains
!----------------------------------------------------------------
subroutine slantstack3d_init(glocs_i,nt_i,ntr_i,np_i,&
     dt_i,dp_i,ot_i,op_i,para)
  real,dimension(:,:),pointer  :: glocs_i 
  integer    :: np_i(2),ntr_i,nt_i
  real       :: dt_i,dp_i(2)
  real       :: ot_i,op_i(2)
  real       :: cen_i(2)
  logical:: para
  pi=acos(-1.)

  ntr=ntr_i
  nt=nt_i

  dt=dt_i
  ot=ot_i

  npx=np_i(1)
  npy=np_i(2)

  dpx=dp_i(1)
  dpy=dp_i(2)

  opx=op_i(1)
  opy=op_i(2)

  glocs=>glocs_i
  if (.not.para)then
     xcen=( maxval(glocs(1,:))-minval(glocs(1,:)) )/2 +minval(glocs(1,:))
     ycen=( maxval(glocs(2,:))-minval(glocs(2,:)) )/2 +minval(glocs(2,:))
  else
     call from_param("xcen",xcen);     call from_param("ycen",ycen);
  end if
  write(0,*) 'Array    min=(',minval(glocs(1,:)),',',minval(glocs(2,:)),')'
  write(0,*) 'Array    min=(',maxval(glocs(1,:)),',',maxval(glocs(2,:)),')'
  write(0,*) 'Array center=(',xcen,',',ycen,')'
  write(0,*) '*************************************************'

end subroutine slantstack3d_init
!----------------------------------------------------------------
integer function slantstack3d_lop(adj,add,modl,dat)
  logical,            intent(in)::adj,add
  real,             dimension(:)::        modl,dat

  integer::ipx,ipy

  slantstack3d_lop=1
  if (adj) then
     if(.not.add) modl=0.
!$OMP PARALLEL DO PRIVATE( ipx,ipy)
       pya:do ipy=1,npy
          pxa:do ipx=1,npx
             call slantadj(ipx,ipy,modl,dat)
          end do pxa
          if(mod(ipy-1,npy/10)==0) write(0,*)'  makeing model ....'
       end do pya
!$OMP END PARALLEL DO 
  else
     if(.not.add) dat=0.
!$OMP PARALLEL DO PRIVATE( ipx,ipy)
     pyf:do ipy=1,npy
        pxf:do ipx=1,npx
           call slantfor(ipx,ipy,modl,dat)
        end do pxf
        if(mod(ipy-1,npy/10)==0) write(0,*) '  makeing data ....'
     end do pyf
!$OMP END PARALLEL DO 
  end if
  slantstack3d_lop=0
end function slantstack3d_lop
!----------------------------------------------------------------
subroutine slantadj(ipx,ipy,modl,dat)
  real,dimension(nt,ntr),intent(in)::dat
  real,dimension(nt,npx,npy),intent(inout)::modl
  integer,intent(in)::ipx,ipy
  real:: px,py,t,z,x,y
  integer::itr,ip,it,iz

  px=opx+(ipx-1)*dpx
  py=opy+(ipy-1)*dpy        
  trace:do itr=1,ntr
     x=glocs(1,itr)-xcen
     y=glocs(2,itr)-ycen
     tau:do iz=1,nt-1
        z= ot + iz*dt
        t= z + px*x + py*y
        it=1.5+ (t-ot)/dt
        if ((it.lt.nt).and.(it.gt.1))then
           modl(iz,ipx,ipy)=modl(iz,ipx,ipy)+ dat(it,itr) 
        end if
     end do tau
  end do trace
end subroutine slantadj
!----------------------------------------------------------------
subroutine slantfor(ipx,ipy,modl,dat)
  real,dimension(nt,ntr),intent(inout)::dat
  real,dimension(nt,npx,npy),intent(in)::modl
  integer,intent(in)::ipx,ipy
  real:: px,py,t,z,x,y
  integer::itr,ip,it,iz

  px=opx+(ipx-1)*dpx
  py=opy+(ipy-1)*dpy        
  trace:do itr=1,ntr
     x=glocs(1,itr)-xcen
     y=glocs(2,itr)-ycen
     tau:do iz=1,nt-1
        z= ot + iz*dt
        t= z + px*x + py*y
        it=1.5+ (t-ot)/dt
        if ((it.lt.nt).and.(it.gt.1))then
           dat(it,itr)=dat(it,itr) + modl(iz,ipx,ipy)
        end if
     end do tau
  end do trace
end subroutine slantfor
!----------------------------------------------------------------
end module pxpytime
